// Copyright (C) 2025 Apple Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
// BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
// THE POSSIBILITY OF SUCH DAMAGE.

#if ENABLE_SWIFTUI && compiler(>=6.0)

import Foundation

@MainActor
protocol PlatformTextSearching {
    associatedtype Interaction: PlatformFindInteraction

    var isFindNavigatorVisible: Bool { get }

    var findInteraction: Interaction? { get }
}

@MainActor
protocol PlatformFindInteraction {
    func presentFindNavigator(showingReplace: Bool)

    func dismissFindNavigator()
}

#if os(macOS)

@MainActor
struct NSTextFinderAdapter: PlatformFindInteraction {
    let wrapped: NSTextFinder

    func presentFindNavigator(showingReplace: Bool) {
        if showingReplace {
            wrapped.performAction(.showReplaceInterface)
        } else {
            wrapped.performAction(.showFindInterface)
        }
    }

    func dismissFindNavigator() {
        wrapped.performAction(.hideFindInterface)
    }
}

#else

@MainActor
struct UIFindInteractionAdapter: PlatformFindInteraction {
    let wrapped: UIFindInteraction
    
    func presentFindNavigator(showingReplace: Bool) {
        wrapped.presentFindNavigator(showingReplace: showingReplace)
    }

    func dismissFindNavigator() {
        wrapped.dismissFindNavigator()
    }
}

#endif

#endif
